#ifndef RENDERERFIXED_H
#define RENDERERFIXED_H

#include "renderer.h"

class RendererFixed : public Renderer
{
	public:

		RendererFixed(void) : angle(0.0f) { }

		virtual void update(float dt)
		{
			this->angle += 90.0f * dt;
		}

		virtual void beginRender(int width, int height)
		{
			// setup projection matrix
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			gluPerspective(60.0f, float(width) / float(height), 0.1f, 10.0f);

			// setup view matrix
			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();

			glEnable(GL_DEPTH_TEST);

			glEnable(GL_LIGHTING);
			{
				const GLfloat sceneAmbient[4] = { 0.2f, 0.2f, 0.2f, 1.0f };
				glLightModelfv(GL_LIGHT_MODEL_AMBIENT, sceneAmbient);

				glEnable(GL_LIGHT0);
				{
					// specify light position with identity modelview matrix (make a "headlight")
					const GLfloat light0Position[4] = { 0.0f, 0.0f, 1.0f, 0.0f };
					glLightfv(GL_LIGHT0, GL_POSITION, light0Position);

					const GLfloat light0Ambient[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
					glLightfv(GL_LIGHT0, GL_AMBIENT, light0Ambient);

					const GLfloat light0Diffuse[4] = { 1.0f, 1.0f, 1.0f, 1.0f };
					glLightfv(GL_LIGHT0, GL_DIFFUSE, light0Diffuse);

					const GLfloat light0Specular[4] = { 1.0f, 1.0f, 1.0f, 1.0f };
					glLightfv(GL_LIGHT0, GL_SPECULAR, light0Specular);
				}

				glEnable(GL_NORMALIZE);
			}

			gluLookAt(0.0f, 0.0f, 2.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);

			glPushMatrix();
			glRotatef(this->angle, 0.0f, 1.0f, 0.0f);
		}

		virtual void render(const Mesh & m)
		{
			glPushMatrix();

				// setup world (model, object) matrix
				glTranslatef(m.position[0], m.position[1], m.position[2]);

				// setup material
				{
					const float matEmission[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
					glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, matEmission);

					//const float matAmbient[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
					const float * matAmbient = m.color;
					glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, matAmbient);

					//const float matDiffuse[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
					const float * matDiffuse = m.color;
					glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, matDiffuse);

					const float matSpecular[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
					glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, matSpecular);

					const float matShininess[1] = { 0.0f };
					glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, matShininess);
				}

				m.render();

			glPopMatrix();
		}

		virtual void endRender(void)
		{
			glPopMatrix();

			glDisable(GL_DEPTH_TEST);
			glDisable(GL_LIGHTING);
			glDisable(GL_LIGHT0);
			glDisable(GL_NORMALIZE);
		}

	protected:

		float angle;
};

#endif // RENDERERFIXED_H
